import hashlib

class Datasets:
  def __init__(self):
    self.by_index = []
    self.by_token = {}
    self.by_name  = {}

  def __getitem__(self,key):
    if isinstance(key, basestring):
      if key in self.by_token:
        return self.by_token[key].data

      hkey = hashlib.sha256(key).hexdigest()
      if hkey in self.by_name:
        return self.by_name[hkey].data

      # raise
      raise KeyError(key)

    else:
      if key >= 0 and key <= len(self.by_index):
        return self.by_index[key].data

      # raise
      raise IndexError("list index out of range")

  def add_dataset(self, ds):
    self.by_index.append(ds)
    self.by_token[ds.token] = ds
    self.by_name[hashlib.sha256(ds.name).hexdigest()] = ds

  def add_legacy_dataset(self):
    self.by_index.append(ds)

  def _repr_html_(self):
    output = (
      "<table class='dataframe'>\n"
      "<thead>\n"
      "<tr style='text-align: right'>"
      "<th>index</th>"
      "<th>qref</th>"
      "<th>name</th>"
      "<th>rows</th>"
      "<th>columns</th>"
      "</tr></thead>\n"
      "<tbody>\n")

    for i, ds in enumerate(self.by_index):
      rows, cols = ds.data.shape
      style = "text-color: #ccc" if ds.error else ""
      if ds.error:
        output += "<tr style='{}'><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td></tr>\n".format(
          style, i, ds.token, ds.name, rows, cols)
      else:
        output += "<tr><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td></tr>\n".format(
          i, ds.token, ds.name, rows, cols)

    output += "</tbody>\n</table>"
    return output
